<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: Criteria.php 4154 2006-02-01 11:38:40Z andrew@m3.net $
*/

require_once MAX_PATH . '/lib/max/Maintenance.php';
require_once MAX_PATH . '/lib/max/Maintenance/OperationInterval.php';

/**
 * @package    MaxMaintenance
 * @subpackage Priority
 * @author     Andrew Hill <andrew@m3.net>
 * @author     James Floyd <james@m3.net>
 */
class MAX_Maintenance_Priority_Criteria
{
    var $rules = array();
    var $operationGroups = array();
    var $oOperationInt;

    /* constructor data should be one or more rows from table acls */
    function MAX_Maintenance_Priority_Criteria($aDeliveryLimitations)
    {
        $this->oOperationInt = &$this->_getOperationInterval();
        // For each $aOptions
        $groupNumber = 0;
        if (is_array($aDeliveryLimitations) && (count($aDeliveryLimitations) > 0)) {
            foreach ($aDeliveryLimitations as $k => $aDeliveryLimitation) {
                // Instantiate appropriate limitation class for limitation type
                $this->rules[$k] = &MAX_Maintenance_Priority_DeliveryLimitation::factory($aDeliveryLimitation);
                // Group logical limitations, split by OR
                //if (strtolower($this->rule[$k]->logical) == 'or') {
                //    $groupNumber++;
                //}
                // Filter out non date/time constraints
                // @todo add single TYPE check - add new delivery limitation subclass to be parent of all below
                if (is_a($this->rules[$k], 'MAX_Maintenance_Priority_DeliveryLimitation_Date')
                        || is_a($this->rules[$k], 'MAX_Maintenance_Priority_DeliveryLimitation_Hour')
                        || is_a($this->rules[$k], 'MAX_Maintenance_Priority_DeliveryLimitation_Day')) {
                    $this->operationGroups[$groupNumber][] = & $this->rules[$k];
                }
            }
        }
    }

    function &_getOperationInterval()
    {
        return OperationInterval::singleton();
    }

	function getActiveAdOperationIntervals(
        $campaignRemainingIntervals,
        $currentOI,
        $campignExpiryDate)
	{
        // are acls activated in conf - if not return campaign OI count
        if ($GLOBALS['_MAX']['CONF']['delivery']['acls'] != 1) {
            return $campaignRemainingIntervals;
        }

        // if delivery delivery limitation exist ...
        if ($this->_getOperationGroupCount() > 0) {

            $inactiveOIs = $this->getInactiveOperationIntervals($currentOI['start'],
                $campignExpiryDate);

            // calculate active operation intervals
            $campaignRemainingIntervals = $campaignRemainingIntervals - $inactiveOIs;
        }
        return $campaignRemainingIntervals;
	}

	function _getOperationGroupCount()
	{
	    return count($this->operationGroups);
	}

   /**
    * Method to check if delivery limitations prevent delivery
    * for the a given operation interval.
    *
    * Only single operation interval ever evaluated by this function,
    * therefore operates are performed on first bit only.  Only one operation
    * group ever exists in current implementation, becuase limitations
    * will consist of all AND or all OR operators.
    *
    * @param array $aCurrentOI keys 'start' & 'end' containing PEAR::Date objects
    * @return bool true is limited or restrictd, false no restriction
    */
    function isLimitedNow($aCurrentOI)
    {
        $dateRangeMaster = $this->oOperationInt->getDayRange(
        	$aCurrentOI['start'],
            $aCurrentOI['end']
        );

         // each logical group of limitations - multiple groups are not implemented
        foreach ($this->operationGroups as $gk => $group) {
            $dateRange = $dateRangeMaster;

            // set first bit - by default
            $dateRange[0][1] = pow(2,0);

            // for each limitation
            foreach ($group as $lk => $limitation) {
                // if isIntervalActive() return false (do not deliver) then set bit
                if (!$this->isIntervalActive($this->operationGroups[$gk][$lk],
                        $aCurrentOI['start'])) {
                    // set first bit as only checking a single date operation interval
                    if ($this->operationGroups[$gk][$lk]->logical == 'or') {
                        // bitwise OR will always set the bit
                        $dateRange[0][1] = pow(2,0);
                    } else {
                        $dateRange[0][1] &= pow(2,0);
                    }
                } else {
                    if ($this->operationGroups[$gk][$lk]->logical == 'and') {
                        $dateRange[0][1] = 0;
                    }
                }
            }
#            // store group results - multiple groups are not implemented
#            $holdGroupResults[] = $dateRange;
        }

#        $finalResult = $this->evaluateRuleGroups($dateRange); - multiple groups are not implemented
        // Find total number of operation intervals set to not deliver
        return ($this->getNonDeliveryOperationIntervalCount($dateRange) > 0) ? true : false;
    }

    /**
    * @todo test + requires serious optimisation
    *
    */
    function getInactiveOperationIntervals($start, $end)
    {
        // setup array representing life of campaign
        $dateRangeMaster = $this->oOperationInt->getDayRange($start, $end);

        $holdGroupResult = array();

        // each logical group of limitations - not implemented
        foreach ($this->operationGroups as $gk => $group) {

            // for each limitation
            foreach ($group as $lk => $limitation) {

                // copy new range set to be evaluated for each limitation
                $dateRange = $dateRangeMaster;

                // for each day in interval
                // where $day[0] = date object - $day[1] = bitwise representation of days OIs
                foreach ($dateRange as $dk => $day) {

                    // initiate flag for interval to 1 - not deliver
                    $dateRange[$dk][1] = pow(2,0);

                    // get current operation interval in day being evaluated
                    $oi = $this->oOperationInt->getDayIntervalNumber($day[0]);

                    // for each OI within current day test delivery limitation
                    for ($oi; $oi < $this->oOperationInt->intervalsPerDay(); $oi++) {

                        // if result of test == 1 (no not deliver) then add to bitwise
                        if (!$this->isIntervalActive($this->operationGroups[$gk][$lk], $day[0])) {
                            // set bit representing the current OI
                            if ($this->operationGroups[$gk][$lk]->logical == 'or') {
                                // Bits that are set in either $a or $b are set.
                                $dateRange[$dk][1] |= (pow(2, $oi));
                            } else {
                                //Bits that are set in both $dateRange[$dk][1] and (pow(2, $oi) are set.
                                $currentBit = ($dateRange[$dk][1] & (pow(2, $oi)));
                                // maintain bits set in $dateRange[$dk][1] and include result of $currentBit
                                $dateRange[$dk][1] |= $currentBit;
                            }
                        } else {
                            if ($this->operationGroups[$gk][$lk]->logical == 'and') {
                                $dateRange[$dk][1] = 0;
                            }
                        }
                        // increment date pointer to next OI
                        $day[0]->addSeconds($this->oOperationInt->secondsPerInterval());
                    }
                    // rewind date pointer to be start of new day
                    $day[0]->subtractSeconds($this->oOperationInt->secondsPerInterval());
                }
            }
            // store group results
            $holdGroupResults[] = $dateRange;
        }

        // Evaluate each group results to get final result by
        // comparing each group results with bitwise & (AND)
        $finalResult = $this->evaluateRuleGroups($holdGroupResults);

        // Find total number of operation intervals set to not deliver
        return $this->getNonDeliveryOperationIntervalCount($finalResult);
    }

   /**
    * Combine bitwise result of multiple limitation groups
    *
    */
    function evaluateRuleGroups($aHoldGroupResults)
    {
        if ((count($aHoldGroupResults) > 0) && (is_array($aHoldGroupResults))) {
            foreach ($aHoldGroupResults as $groupResult) {
                if (!isset($finalResult)) {
                    $finalResult = $groupResult;
                } else {
                    foreach ($groupResult as $k => $day) {
                        $finalResult[$k][1] &= $day[1];
                    }
                }
            }
        } else {
            $finalResult = array();
        }
        return $finalResult;
    }

//    function isIntervalInactive($oLimitationDate, $oDate)
//    {
//        return ($oLimitationDate->testInterval($oDate) == 1) ? true : false;
//    }

    function isIntervalActive($oLimitationDate, $oDate)
    {
        return ($oLimitationDate->testInterval($oDate) == 1) ? false : true;
    }

   /**
    * Method to count the number of operation intervals set to
    * not deliver
    *
    *
    */
    function getNonDeliveryOperationIntervalCount($finalResult)
    {
        $n = 0;
        if (is_array($finalResult) && (count($finalResult) > 0)) {
            foreach ($finalResult as $dayResult) {
                $n += $this->getBitSetCount($dayResult[1]);
            }
        }
        return $n;
    }

   /**
    * Method to return the number of bits set for a given
    * integer representation of a binary number
    *
    * @param integer $val integer representation of binary number to be evaluated
    * @return integer count of number of bits set
    */
    function getBitSetCount($val)
    {
        $c = 0;
        $v = (int)$val;
        for ($c=0; $v; $c++) {
           $v &= $v - 1;
        }
        return $c;
    }
}

?>
