<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id:
*/

require_once MAX_PATH . '/lib/max/Maintenance/Priority/Criteria.php';
require_once MAX_PATH . '/lib/max/Maintenance/Priority/DeliveryLimitation.php';
require_once MAX_PATH . '/lib/pear/Date.php';

/**
 * A class for testing the Maintenance_Priority_Criteria class.
 *
 * @package    MaxMaintenance
 * @subpackage TestSuite
 * @author     James Floyd<james@m3.net>
 */
class Maintenance_TestOfPriorityCriteria extends UnitTestCase
{

   /**
    * The constructor method.
    */
    function Maintenance_TestOfPriorityCriteria()
    {
        $this->UnitTestCase();
    }

    /**
     * test 1 - make sure correct delivery limitation objects are being created
     **/
    function testConstructor()
    {
        $aDeliveryLimitations = array(
            array(
                'ad_id' => 1,
                'logical' => 'and',
                'type' => 'Time:Hour',
                'comparison' => '==',
                'data' => '1,7,18,23',
                'executionorder' => 1,
            ),
            array(
                'ad_id' => 1,
                'logical' => 'and',
                'type' => 'Time:Day',
                'comparison' => '==',
                'data' => '1',
                'executionorder' => 2,
            ),
            array(
                'ad_id' => 1,
                'logical' => 'and',
                'type' => 'Time:Date',
                'comparison' => '==',
                'data' => '2005:10:10 00:00:00',
                'executionorder' => 3,
            ),
            array(
                'ad_id' => 1,
                'logical' => 'and',
                'type' => 'ip',
                'comparison' => '==',
                'data' => '192.168.22.5',
                'executionorder' => 4,
            ),
        );

        Mock::generatePartial('MAX_Maintenance_Priority_Criteria', $class = 'MAX_Maintenance_Priority_Criteria' . rand(),
            array('_getOperationInterval'));
        $obj =& new $class($this);
        $obj->setReturnValue('_getOperationInterval', true);
        // call constructor to test
        $obj->MAX_Maintenance_Priority_Criteria($aDeliveryLimitations);

        $this->assertTrue(count($obj->rules) == 4);

        // only ever one group of delivery limitations
        // Test that only 3 of 4 delivery limitation objects are added to group
        $this->assertTrue(count($obj->operationGroups[0]) == 3);

        // check correct object are created, not really part of this method
        $this->assertTrue(is_a($obj->operationGroups[0][0], 'MAX_Maintenance_Priority_DeliveryLimitation_Hour'));
        $this->assertTrue(is_a($obj->operationGroups[0][1], 'MAX_Maintenance_Priority_DeliveryLimitation_Day'));
        $this->assertTrue(is_a($obj->operationGroups[0][2], 'MAX_Maintenance_Priority_DeliveryLimitation_Date'));
    }

    function test_getOperationInterval()
    {
        $oCriteria = new MAX_Maintenance_Priority_Criteria(array());

        // test operation interval class iss returned..
        $oOperationInt = & $oCriteria->_getOperationInterval();
        $this->assertTrue(is_a($oOperationInt, 'OperationInterval'));
    }

    function testGetActiveAdOperationIntervals()
    {
        // get global array and hold original ['acls'] value
        // that will be reinstated at end of test method
        $conf = & $GLOBALS['_MAX']['CONF'];
        $holdAcls = $conf['delivery']['acls'];

        // Test 1 - return value if acls is set to false
        $campaignRemainingIntervals = 3000;
        $conf['delivery']['acls'] = false;
        $oCriteria = new MAX_Maintenance_Priority_Criteria(array());
        $currentOI = Maintenance::getNextOperationalInterval();
        $campignExpiryDate = $currentOI[1];
        $this->assertEqual($campaignRemainingIntervals, $oCriteria->getActiveAdOperationIntervals($campaignRemainingIntervals, $currentOI, $currentOI[1]));

        // Test 2 - test the correct remaining interval calculation is being performed
        $conf['delivery']['acls'] = 1;
        Mock::generatePartial('MAX_Maintenance_Priority_Criteria', 'MockCriteria', array('getInactiveOperationIntervals', '_getOperationGroupCount'));
        $oCriteria = &new MockCriteria($this);
        // set return for number of inactive OIs
        $oCriteria->setReturnValue('getInactiveOperationIntervals', 100);
        // Mock a method so a number of delivery limitations exist
        $oCriteria->setReturnValue('_getOperationGroupCount', 1);
        $this->assertEqual(2900, $oCriteria->getActiveAdOperationIntervals($campaignRemainingIntervals, $currentOI, $currentOI[1]));

        // reinstate original ['acls'] value
        $conf['acls'] = $holdAcls;
    }

    function testIsLimitedNow()
    {
        // hack the test, see what the results are
        Mock::generatePartial('MAX_Maintenance_Priority_Criteria',
            $class = 'MAX_Maintenance_Priority_Criteria' . rand(),
            array('_getOperationInterval', 'isIntervalActive', 'getNonDeliveryOperationIntervalCount')
        );

        $obj = & new $class($this);

        // Mock OperationInterval Class
        Mock::generatePartial('OperationInterval', $class = 'OperationInterval' . rand(), array('getDayRange'));
        $oOperationInt = & new $class($this);

        // set up single day to be tested..
        $aDayRange = array(
            array(
                0 => new Date('2005-05-20 00:00:00'),
                1 => 0,
            )
        );

        $oOperationInt->setReturnValue('getDayRange', $aDayRange);

        $obj->setReturnValue('_getOperationInterval', $oOperationInt);
        $obj->setReturnValue('isIntervalActive', false);
//        $obj->setReturnValue('getNonDeliveryOperationIntervalCount', true);

         $aDeliveryLimitations = array(
            array(
                'ad_id' => 1,
                'logical' => 'and',
                'type' => 'Time:Hour',
                'comparison' => '==',
                'data' => '1,7,18,23',
                'executionorder' => 1,
            )
        );
        // set up delivery limitations to be used.. These are not mocks, hack!
        $obj->MAX_Maintenance_Priority_Criteria($aDeliveryLimitations);
        $this->assertFalse($obj->isLimitedNow(array('start' => new Date(), 'end' => new Date())));

    }

    function testGetInactiveOperationIntervals()
    {
        // use integration test..
    }

   /**
    * Test that groups of delivery (0) and non-delivery (1)
    * bitwise flags are being correctly evaluated
    */
    function testEvaluateRuleGroups()
    {
        $aGroupResults = array(
            array(
                // first bit set
                array(
                    1 => 1,
                ),
                // 1st, 3rd bit set
                array(
                    1 => 5,
                ),
                // 7th bit set
                array(
                    1 => 64,
                ),
            ),
            array(
                // 1st, 8th bit set
                array(
                    1 => 129,
                ),
                // 1st, 3rd, 4th, 5th, 6th bit set
                array(
                    1 => 61,
                ),
                // 4th and 5th, 7th bit set
                array(
                    1 => 88,
                ),
            )
        );
        $oCriteria = new MAX_Maintenance_Priority_Criteria(array());
        $results = $oCriteria->evaluateRuleGroups($aGroupResults);

        // test value of first array
        $this->assertTrue($results[0][1] == 1);

        // test value of 2nd array
        $this->assertTrue($results[1][1] == 5);

        // test value of 3rd array
        $this->assertTrue($results[2][1] == 64);
    }

    function testIsIntervalActive()
    {
        // test 1 - test interval returns 1 = do not deliver
        Mock::generatePartial('MAX_Maintenance_Priority_DeliveryLimitation_Date', $class = 'MAX_Maintenance_Priority_DeliveryLimitation_Date' . rand(),
            array('testInterval'));
        $obj = & new $class($this);
        $obj->setReturnValue('testInterval', 1, array('returnTrue'));
        $obj->setReturnValue('testInterval', 0, array('returnFalse'));

        // create object to test
        $oCriteria = new MAX_Maintenance_Priority_Criteria(array());
        // run method to test
        $this->assertTrue($oCriteria->isIntervalActive($obj, 'returnFlase'));
        $this->assertFalse($oCriteria->isIntervalActive($obj, 'returnTrue'));
    }

    function testGetNonDeliveryOperationIntervalCount()
    {
        // setup test input parameter array
        // only concerned with $result[n][1]
        $oneBitSet = pow(2,0);
        $twoBitSet = $oneBitSet |  pow(2,1);
        $fourBitSet = $twoBitSet |  pow(2,2) | pow(2,3);
        $result = array(
                    array(0, $oneBitSet),
                    array(0, $twoBitSet ),
                    array(0, $fourBitSet),
                    array(0, $fourBitSet),
        );
        // (1 + 2 + 4 + 4) = 11 bits set in total
        $oCriteria = new MAX_Maintenance_Priority_Criteria(array());
        $this->assertEqual(11, $oCriteria->getNonDeliveryOperationIntervalCount($result));

        // Test 2 - test zero return value when no input
        $this->assertEqual(0, $oCriteria->getNonDeliveryOperationIntervalCount(array()));
    }

    function testGetBitSetCount()
    {
        $val = 0;
        // Test no bits set
        $this->assertEqual($val, MAX_Maintenance_Priority_Criteria::getBitSetCount(0));
        // Test one bit set - set 1st bit
        $val |= pow(2, 0);
        $this->assertEqual(1, MAX_Maintenance_Priority_Criteria::getBitSetCount($val));
        // Test two bit set - set 1st and 11th bit
        $val |= pow(2, 10);
        $this->assertEqual(2, MAX_Maintenance_Priority_Criteria::getBitSetCount($val));
        // Test three bit set - set 1st, 10th and 11th bit
        $val |= pow(2, 9);
        $this->assertEqual(3, MAX_Maintenance_Priority_Criteria::getBitSetCount($val));
    }

    function testBitwiseFlagLogic()
    {
        // test AND operator, all inactive group
        $aLimitations = array(
            array('active' => false, 'logic' => 'and',),
            array('active' => false, 'logic' => 'and',),
            array('active' => false, 'logic' => 'and',),
            array('active' => false, 'logic' => 'and',),
        );
        $this->assertTrue($this->biwiseLogic($aLimitations) == 1);


        // test AND operator, first delivery limitation active
        $aLimitations = array(
            array('active' => true,  'logic' => 'and',),
            array('active' => false, 'logic' => 'and',),
            array('active' => false, 'logic' => 'and',),
            array('active' => false, 'logic' => 'and',),
        );
        $this->assertTrue($this->biwiseLogic($aLimitations) == 0);

        // test AND operator, middle delivery limitation active
        $aLimitations = array(
            array('active' => false, 'logic' => 'and',),
            array('active' => false, 'logic' => 'and',),
            array('active' => true,  'logic' => 'and',),
            array('active' => false, 'logic' => 'and',),
        );
        $this->assertTrue($this->biwiseLogic($aLimitations) == 0);

        // test AND operator, last delivery limitation active
        $aLimitations = array(
            array('active' => false, 'logic' => 'and',),
            array('active' => false, 'logic' => 'and',),
            array('active' => false, 'logic' => 'and',),
            array('active' => true,  'logic' => 'and',),
        );
        $this->assertTrue($this->biwiseLogic($aLimitations) == 0);

        // test OR operator, all inactive group
        $aLimitations = array(
            array('active' => false, 'logic' => 'or',),
            array('active' => false, 'logic' => 'or',),
            array('active' => false, 'logic' => 'or',),
            array('active' => false, 'logic' => 'or',),
        );
        $this->assertTrue($this->biwiseLogic($aLimitations) == 1);

        // test OR operator, all inactive group
        $aLimitations = array(
            array('active' => true, 'logic' => 'or',),
            array('active' => false, 'logic' => 'or',),
            array('active' => false, 'logic' => 'or',),
            array('active' => false, 'logic' => 'or',),
        );
        $this->assertTrue($this->biwiseLogic($aLimitations) == 1);

        // test OR operator, all inactive group
        $aLimitations = array(
            array('active' => false, 'logic' => 'or',),
            array('active' => false, 'logic' => 'or',),
            array('active' => false, 'logic' => 'or',),
            array('active' => true, 'logic' => 'or',),
        );
        $this->assertTrue($this->biwiseLogic($aLimitations) == 1);

    }

    function biwiseLogic($aLimitations)
    {
        // initialise bitwise to not deliver (1)
        $bitwise = pow(2,0);
        foreach($aLimitations as $aLimitation) {
            // delivery restriction
            if (!$aLimitation['active']) {
                if ($aLimitation['logical'] == 'or') {
                    $bitwise = pow(2,0);
//                    return true;
                } else {
                    $bitwise &= pow(2,0);
                }
            } else {
                // not restricted active..
                // test to see if this works
                if ($aLimitation['logic'] == 'and') {
                    $bitwise = 0;
                }
            }
        }
        return $bitwise;
    }

}

?>
