<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: PriorityDeliveryLimitationHour.mtp.test.php 4131 2006-01-26 18:08:38Z andrew@m3.net $
*/

require_once MAX_PATH . '/lib/max/Maintenance/Priority/DeliveryLimitation.php';
require_once MAX_PATH . '/lib/max/Maintenance/Priority/DeliveryLimitation/Hour.php';
require_once 'Date.php';

/**
 * @package    MaxMaintenance
 * @subpackage TestSuite
 * @author     James Floyd <james@m3.net>
 */
class Delivery_TestOfPriorityDeliveryLimitationHour extends UnitTestCase
{
   /**
    * The constructor method.
    */
    function Delivery_TestOfPriorityDeliveryLimitationHour()
    {
        $this->UnitTestCase();
    }

   /**
    * Test 1 -  correct non delivery time restrictions are set
    *
    */
    function testCalculateNonDeliveryRestrictions()
    {
        // Test 1 - test date == delivery requirement
        $aLimitation = array(
            'ad_id' => 1,
            'logical' => 'and',
            'type' => 'Time:Hour',
            'comparison' => '==',
            'data' => '1,7,18,23',
            'executionorder' => 1,
        );
        $hours = range(0,23);
        unset($hours[1],$hours[7],$hours[18],$hours[23]);
        $oLimitationHour = & MAX_Maintenance_Priority_DeliveryLimitation::factory($aLimitation);
        $this->assertEqual($oLimitationHour->data, $hours);
        $this->assertTrue(count($oLimitationHour->data) == 20);
        unset($aLimitation, $oLimitationHour,$hours);

        // Test 1 - test date == delivery requirement
        $aLimitation = array(
            'ad_id' => 1,
            'logical' => 'and',
            'type' => 'Time:Hour',
            'comparison' => '==',
            'data' => '0,23',
            'executionorder' => 1,
        );
        $hours = range(0,23);
        unset($hours[0],$hours[23]);
        $oLimitationHour = & MAX_Maintenance_Priority_DeliveryLimitation::factory($aLimitation);
        $this->assertEqual($oLimitationHour->data, $hours);
        $this->assertTrue(count($oLimitationHour->data) == 22);
        unset($aLimitation, $oLimitationHour,$hours);
    }

   /**
    * Test 1 - return value of test interval for set of
    * non delivery restrictions
    *
    */
    function testTestInterval()
    {
        Mock::generatePartial('MAX_Maintenance_Priority_DeliveryLimitation_Hour', $mockClass = 'MAX_Maintenance_Priority_DeliveryLimitation_Hour' . rand(),
            array('calculateNonDeliveryRestrictions'));
        $obj =& new $mockClass($this);

        $obj->setReturnValue('calculateNonDeliveryRestrictions', true);

        $aLimitation = array(
            'ad_id' => 1,
            'logical' => 'and',
            'type' => 'Time:Hour',
            'comparison' => '==',
            'data' => '1, 5, 7, 20',
            'executionorder' => 1,
        );
        $aData = range(0,23);
        unset($aData[1], $aData[5], $aData[7], $aData[20]);
        $obj->MAX_Maintenance_Priority_DeliveryLimitation_Hour($aLimitation);
        // set fake delivery restrictions that should be set by calculateNonDeliveryRestrictions()
        $obj->data = $aData;

        $oTestStartDate = new Date('2005-01-01 00:00:00');

        foreach (range(0,23) as $key => $hour) {
            if (in_array($key, $aData)) {
                $this->assertTrue($obj->testInterval($oTestStartDate) == 1);
            } else {
                $this->assertTrue($obj->testInterval($oTestStartDate) == 0);
            }
            // move test hour on by one hour
            $oTestStartDate->addSeconds(60 * 60);
        }
    }

   /**
    * Test if method returns expect time period
    */
    function testMinutesPerTimePeriod()
    {
        $this->assertEqual(MAX_Maintenance_Priority_DeliveryLimitation_Hour::minutesPerTimePeriod(), 60);
    }
}

?>
